unit imdb;

interface

uses
	Windows, Forms, Variants, ComObj, Classes, SysUtils, Dialogs, ActiveX;

type
    TMdbSeek = class(TThread)
    private
        ProgressMsg	: string;
        Progress	: TStrings;
        MdbFiles	: TStrings;
        Works		: TStrings;
        procedure	SyncShowForm();
        procedure	ShowForm(AMsg: string);
	public
		constructor	Create(AMdbFiles: TStrings; AProgress: TStrings = nil);
        destructor	Destroy; override;
    protected
        procedure	Execute; override;
    end;

var
    MdbSeeker: TMdbSeek;

implementation

{ TMdbSeek }

(*******************************************************************************

	TableDef.Field ֘Ȁ

*******************************************************************************)

(*
	=============================================
    TableDef.Fieldp\
	=============================================
*)
type
    TTableDefField = record
        AllowZeroLength : Boolean;
        Attributes      : string;
        DefaultValue    : string;
        Name            : string;
        OrdinalPosition : integer;
        Required        : Boolean;
        Size            : integer;
        SourceField     : string;
        SourceTable     : string;
        FieldType       : string;
        Error			: string;
    end;

(*
	=============================================
	TableDef.Field.Attributes萔
	=============================================
*)
const
    dbFixedField		= $00000001;
    dbVariableField		= $00000002;
    dbAutoIncrField		= $00000010;
    dbUpdatableField	= $00000020;
    dbSystemField		= $00002000;
    dbHyperlinkField	= $00008000;
    dbDescending		= $00000001;

(*
	=============================================
	function FieldAttrToStr(Attr: Long): string;
	=============================================
*)
function FieldAttrToStr(Aattr: integer): string;
begin
    Result := '';
    if (Aattr and dbFixedField)		<> 0 then Result := Result + ',TCYŒ';
    if (Aattr and dbVariableField)	<> 0 then Result := Result + ',TCY';
    if (Aattr and dbAutoIncrField)  <> 0 then Result := Result + ',I[gio[';
    if (Aattr and dbUpdatableField) <> 0 then Result := Result + ',tB[hlύX\';
    if (Aattr and dbSystemField)   	<> 0 then Result := Result + ',vP[V܂';
    if (Aattr and dbHyperlinkField) <> 0 then Result := Result + ',nCp[N܂';
    if (Aattr and dbDescending)    	<> 0 then Result := Result + ',~\[g';
    Delete(Result, 1, 1);
end;

(*
	=============================================
	Property.Type萔
	=============================================
*)
const
    dbBoolean	= $00000001;
    dbByte		= $00000002;
    dbInteger	= $00000003;
    dbLong		= $00000004;
    dbCurrency	= $00000005;
    dbSingle	= $00000006;
    dbDouble	= $00000007;
    dbDate		= $00000008;
    dbBinary	= $00000009;
    dbText		= $0000000A;
    dbLongBinary = $0000000B;
    dbMemo		= $0000000C;
    dbGUID		= $0000000F;
    dbBigInt	= $00000010;
    dbVarBinary	= $00000011;
    dbChar		= $00000012;
    dbNumeric	= $00000013;
    dbDecimal	= $00000014;
    dbFloat		= $00000015;
    dbTime		= $00000016;
    dbTimeStamp	= $00000017;

(*
	=============================================
	function PropertyTypeToStr(Prop: integer): string;
	=============================================
*)
function PropertyTypeToStr(APropType: integer): string;
begin
	case APropType of
		dbBoolean	: Result := 'u[^(Boolean)';
		dbByte		: Result := 'oCg^(Byte)';
        dbInteger	: Result := '^(Integer)';
        dbLong		: Result := '^(Long)';
        dbCurrency	: Result := 'ʉ݌^(Currency)';
        dbSingle	: Result := 'Px_^(Single)';
        dbDouble	: Result := '{x_^(Double)';
        dbDate		: Result := 't/^(Date/Time)';
        dbBinary	: Result := 'oCi^(Binary)';
        dbText		: Result := 'eLXg^(Text)';
        dbLongBinary: Result := 'OoCi^(LongBinary)-OLEIuWFNg^(OLE Object)';
        dbMemo		: Result := '^(Memo)';
        dbGUID		: Result := 'GUID^(GUID)';
        dbBigInt	: Result := 'Big Integer ^ (Big Integer)';
        dbVarBinary	: Result := 'ϒoCi^(VarBinary)';
        dbChar		: Result := 'CHAR^(Char)';
        dbNumeric	: Result := 'Numeric^(Numeric)';
        dbDecimal	: Result := '10i^(Decimal)';
        dbFloat		: Result := '_^(Float)';
        dbTime		: Result := '^(Time)';
        dbTimeStamp	: Result := '^CX^v^(TimeStamp)';
    	else		  Result := 'UnKnown(' + IntToStr(APropType) + ')'
	end;
end;

(*
	=============================================
	function GetTableDefField(Adb: Variant; ATblIdx, AFldIdx: integer): TTableDefField;
	=============================================
*)
function GetTableDefField(Adb: Variant; ATblIdx, AFldIdx: integer): TTableDefField;
begin
	with Result do
    begin
		try
            AllowZeroLength :=                   Adb.TableDefs[ATblIdx].Fields[AFldIdx].AllowZeroLength;
            Attributes      :=    FieldAttrToStr(Adb.TableDefs[ATblIdx].Fields[AFldIdx].Attributes);
            DefaultValue    :=                   Adb.TableDefs[ATblIdx].Fields[AFldIdx].DefaultValue;
            Name            :=                   Adb.TableDefs[ATblIdx].Fields[AFldIdx].Name;
            OrdinalPosition :=                   Adb.TableDefs[ATblIdx].Fields[AFldIdx].OrdinalPosition;
            Required        :=                   Adb.TableDefs[ATblIdx].Fields[AFldIdx].Required;
            Size            :=                   Adb.TableDefs[ATblIdx].Fields[AFldIdx].Size;
            SourceField     :=                   Adb.TableDefs[ATblIdx].Fields[AFldIdx].SourceField;
            SourceTable     :=                   Adb.TableDefs[ATblIdx].Fields[AFldIdx].SourceTable;
            FieldType       := PropertyTypeToStr(Adb.TableDefs[ATblIdx].Fields[AFldIdx].Type);
        except
			on E:Exception do
            begin
				Error := E.Message;
            end;
        end;
	end;
end;

(*
	=============================================
	function TableDefFieldToHTMLTable(AFld: TTableDefField): string;
	=============================================
*)
function TableDefFieldToHTMLTableHead(): string;
begin
	Result :=
	  	'<table cellpadding=3 cellspacing=0 border=1>'
	  +	'<tr>'
      + '<th nowrap>ԍ</th>'
      + '<th nowrap>tB[h</th>'
      + '<th nowrap>^</th>'
      + '<th nowrap></th>'
      + '<th nowrap>󕶎</th>'
      + '<th nowrap>NULLs</th>'
      + '<th nowrap></th>'
      + '<th nowrap>Kl</th>'
      + '<th nowrap>Ne[u</th>'
      + '<th nowrap>NtB[h</th>'
      + '</tr>';
end;

function TableDefFieldToHTMLTableBody(AFld: TTableDefField): string;

	function NullToNbsp(const Str: string): string;
    begin
    	if Str = '' then Result := '&nbsp;' else Result := Str;
    end;

begin
	Result :=
		'<tr>'
      + '<td nowrap align=right>' +   IntToStr(AFld.OrdinalPosition)       + '</td>'
      + '<td nowrap>'             + NullToNbsp(AFld.Name)                  + '</td>'
      + '<td nowrap>'             + NullToNbsp(AFld.FieldType)             + '</td>'
      + '<td nowrap align=right>' +   IntToStr(AFld.Size)                  + '</td>'
      + '<td nowrap>'             +  BoolToStr(AFld.AllowZeroLength, True) + '</td>'
      + '<td nowrap>'             +  BoolToStr(AFld.Required, True)        + '</td>'
      + '<td nowrap>'             + NullToNbsp(AFld.Attributes)            + '</td>'
      + '<td nowrap>'             + NullToNbsp(AFld.DefaultValue)          + '</td>'
      + '<td nowrap>'             + NullToNbsp(AFld.SourceTable)           + '</td>'
      + '<td nowrap>'             + NullToNbsp(AFld.SourceField)           + '</td>'
      + '</tr>';
end;

function TableDefFieldToHTMLTableTail(): string;
begin
	Result := '</table>';
end;

(*******************************************************************************

	TableDef ֘Ȁ

*******************************************************************************)

(*
	=============================================
	TableDefp\
	=============================================
*)
type
    TTableDef = record
        Attributes      : string;
        DateCreated     : string;
        LastUpdated     : string;
        Name            : string;
        RecordCount     : integer;
        SourceTableName : string;
        Fields			: array of TTableDefField;
        Error			: string;
    end;

(*
	=============================================
	TableDef.Attributes萔
	=============================================
*)
const
    dbAttachExclusive	= $00010000;
    dbAttachSavePWD		= $00020000;
    dbSystemObject		= $80000002;
    dbAttachedTable		= $40000000;
    dbAttachedODBC		= $20000000;
    dbHiddenObject		= $00000001;

(*
	=============================================
	function TableDefAttrToStr(Attr: Long): string;
	=============================================
*)
function TableDefAttrToStr(Aattr: integer): string;
begin
    Result := '';
    if (Aattr and dbAttachExclusive) <> 0 then Result := Result + ',r[hŊJNe[u';
    if (Aattr and dbAttachSavePWD)   <> 0 then Result := Result + ',[U[IDƃpX[hڑƋɕۑ郊[ge[u';
    if (Aattr and dbSystemObject)    <> 0 then Result := Result + ',VXee[u';
    if (Aattr and dbHiddenObject)    <> 0 then Result := Result + ',\̃e[u';
    if (Aattr and dbAttachedTable)   <> 0 then Result := Result + ',ODBCf[^\[X̃Ne[u';
    if (Aattr and dbAttachedODBC)    <> 0 then Result := Result + ',ODBCf[^\[X̃Ne[u';
    Delete(Result, 1, 1);
end;

(*
	=============================================
	function GetTableDef(Adb: Variant; Aidx: integer): TTableDef;
	=============================================
*)
function GetTableDef(Adb: Variant; Aidx: integer): TTableDef;
var
	i, Count: integer;
begin
	with Result do
    begin
		try
            Attributes		:= TableDefAttrToStr(Adb.TableDefs[Aidx].Attributes);
            DateCreated		:=          VarToStr(Adb.TableDefs[Aidx].DateCreated);
            LastUpdated		:=          VarToStr(Adb.TableDefs[Aidx].LastUpdated);
            Name			:=                   Adb.TableDefs[Aidx].Name;
            RecordCount		:=                   Adb.TableDefs[Aidx].RecordCount;
            SourceTableName	:=                   Adb.TableDefs[Aidx].SourceTableName;
            Count			:= Adb.TableDefs[Aidx].Fields.Count;
            SetLength(Fields, Count);
            for i := 0 to Count - 1 do
            begin
                Fields[i] := GetTableDefField(Adb, Aidx, i);
            end;
        except
			on E:Exception do
            begin
				Error := E.Message;
            end;
        end;
	end;
end;

(*
	=============================================
	function TableDefToHTMLTable(ATbl: TTableDef): string;
	=============================================
*)
function TableDefToHTMLTableHead(): string;
begin
	Result :=
	  	'<table cellpadding=3 cellspacing=0 border=1>'
	  +	'<tr>'
      + '<th nowrap>e[u</th>'
      + '<th nowrap></th>'
      + '<th nowrap>R[h</th>'
      + '<th nowrap>N</th>'
      + '<th nowrap>쐬</th>'
      + '<th nowrap>XV</th>'
      + '</tr>';
end;

function TableDefToHTMLTableBody(AMdbName: string; ATbl: TTableDef): string;

	function NullToNbsp(const Str: string): string;
    begin
    	if Str = '' then Result := '&nbsp;' else Result := Str;
    end;

const
	TABLE_LINK = '<a href="##MDBNAME#_#TBLNAME#">#TBLNAME#</a>';
var
	tmp: string;
begin
    tmp := TABLE_LINK;
    tmp := StringReplace(tmp, '#MDBNAME#', AMdbName, [rfReplaceAll]);
    tmp := StringReplace(tmp, '#TBLNAME#', ATbl.Name, [rfReplaceAll]);
	Result :=
		'<tr>'
      + '<td nowrap>'             + tmp                              + '</td>'
      + '<td nowrap>'             + NullToNbsp(ATbl.Attributes)      + '</td>'
      + '<td nowrap align=right>' +   IntToStr(ATbl.RecordCount)     + '</td>'
      + '<td nowrap>'             + NullToNbsp(ATbl.SourceTableName) + '</td>'
      + '<td nowrap>'             + NullToNbsp(ATbl.DateCreated)     + '</td>'
      + '<td nowrap>'             + NullToNbsp(ATbl.LastUpdated)     + '</td>'
      + '</tr>';
end;

function TableDefToHTMLTableTail(): string;
begin
	Result := '</table>';
end;

(*******************************************************************************

	mdb ֘Ȁ

*******************************************************************************)

(*
	=============================================
	Mdbt@Cp\
	=============================================
*)
type
    TMdb = record
		Name		: string;
		StartTick	: Cardinal;
		FinishTick	: Cardinal;
        TableDefs	: array of TTableDef;
        Error		: string;
    end;

(*
	=============================================
	procedure OpenMdb();
	=============================================
*)
procedure OpenMdb(var ADBEngine, ADatabase: Variant; AMdbPath: string);
begin
    ADBEngine	:= CreateOleObject('DAO.DBEngine.36');
    ADatabase	:= ADBEngine.Workspaces[0].OpenDatabase(AMdbPath, False, False);
end;
                                                
(*
	=============================================
	procedure CloseMdb();
	=============================================
*)
procedure CloseMdb(var ADBEngine, ADatabase: Variant);
begin
   	ADatabase.Close;
    ADatabase := Unassigned;
    ADBEngine := Unassigned;
end;

(*
	=============================================
	function GetMdb(AMdbPath: string): TMdb;
	=============================================
*)
function GetMdb(AMdbPath: string): TMdb;
var
	daoEngine, daoDB: Variant;
    i, Count: integer;
begin
    Result.StartTick := GetTickCount;
    OpenMdb(daoEngine, daoDB, AMdbPath);
    try try
        Result.Name := ExtractFileName(AMdbPath);
        Count := daoDB.TableDefs.Count;
        SetLength(Result.TableDefs, Count);
        for i := 0 to Count - 1 do
        begin
            Result.TableDefs[i] := GetTableDef(daoDB, i);
        end;
    except
        on E:Exception do
        begin
            Result.Error := E.Message;
        end;
    end;
    finally
		CloseMdb(daoEngine, daoDB);
    end;
    Result.FinishTick := GetTickCount;
end;

(*
	=============================================
	function MdbToHTML(AMdb: TMdb): string;
	=============================================
*)
function MdbToHTML(AMdb: TMdb): string;
const
	MDB_TITLE = '<h1 id="#MDBNAME#"><a href="#">#MDBNAME#</a></h1>';
	TBL_TITLE = '<h2 id="#MDBNAME#_#TBLNAME#"><a href="##MDBNAME#">#TBLNAME# (in #MDBNAME#)</a></h2>';
var
	i, j: integer;
    tmp: string;
    TableDef: TTableDef;
    FieldDef: TTableDefField;
begin
	//------------------------------------------
    tmp := MDB_TITLE;
    tmp := StringReplace(tmp, '#MDBNAME#', AMdb.Name, [rfReplaceAll]);
	Result := tmp;
	Result := Result + TableDefToHTMLTableHead();
	for i := 0 to Length(AMdb.TableDefs) - 1 do
    begin
        TableDef := AMdb.TableDefs[i];
		Result := Result + TableDefToHTMLTableBody(AMdb.Name, TableDef);
    end;
	Result := Result + TableDefToHTMLTableTail();
	//------------------------------------------
	for i := 0 to Length(AMdb.TableDefs) - 1 do
    begin
        TableDef := AMdb.TableDefs[i];
        tmp := TBL_TITLE;
        tmp := StringReplace(tmp, '#MDBNAME#', AMdb.Name, [rfReplaceAll]);
        tmp := StringReplace(tmp, '#TBLNAME#', TableDef.Name, [rfReplaceAll]);
		Result := Result + tmp;
		Result := Result + TableDefFieldToHTMLTableHead();
		for j := 0 to Length(TableDef.Fields) - 1 do
    	begin
            FieldDef := TableDef.Fields[j];
			Result := Result + TableDefFieldToHTMLTableBody(FieldDef);
	    end;
		Result := Result + TableDefFieldToHTMLTableTail();
    end;

end;

(*******************************************************************************

	o

*******************************************************************************)

(*
	=============================================
	function MakeDottedLine(AHead, ATail: string; ATotalLen: integer): string;
	=============================================
*)
function MakeDottedLine(AHead, ATail: string; ATotalLen: integer): string;
var
	Dots: string;
begin
    Dots := StringOfChar('.', ATotalLen - Length(AHead + ATail));
	Result := AHead + Dots + ATail;
end;

(*
	=============================================
	function GetCurrentTime(): string;
	=============================================
*)
function GetCurrentTime(AFormat: string = 'yyyy/mm/dd hh:nn:ss'): string;
begin
	Result := FormatDateTime(AFormat, Now);
end;

procedure SaveHTMLPage(BodyContens: string);
var 
    F: TextFile;
    Path: string;
begin
    Path := ExtractFilePath(Application.ExeName) + GetCurrentTime('yyyymmddhhnnss') + '.html';
    AssignFile(F, Path);
	try
        Rewrite(F);
        Writeln(F, '<html><head><style type="text/css"><!--body,table{font-family:"courier new";}table{padding:3px;}th{padding:1px;text-align:left;background-color:#eeeeee;}--></style></head><body>');
        Writeln(F, BodyContens);
        Writeln(F, '<body></html>');
	finally
        CloseFile(F);
    end;
end;

(*******************************************************************************

	TMdbSeek IMPLIMENTATION

*******************************************************************************)

(*
	=============================================
	constructor	TMdbSeek.Create(AMdbFiles: TStrings);
	=============================================
*)
constructor	TMdbSeek.Create(AMdbFiles: TStrings; AProgress: TStrings = nil);
begin
    inherited Create(True);
	MdbFiles := TStringList.Create;
    MdbFiles.Text := AMdbFiles.Text;
    Progress := AProgress;
    Works := TStringList.Create;
end;

(*
	=============================================
	destructor	TMdbSeek.Destroy;
	=============================================
*)
destructor	TMdbSeek.Destroy;
begin
	MdbFiles.Free;
    Works.Free;
end;

(*
	=============================================
	procedure	TMdbSeek.Execute;
	=============================================
*)
procedure	TMdbSeek.Execute;
var
	i: integer;
    MdbInfos: array of TMdb;
    FileName, Time: string;
begin
    ShowForm(GetCurrentTime() + 'Jn܂B');

    ShowForm('');
    CoInitialize(nil);	// COM̏(STA)

    ShowForm('*****͊Jn*****');
	SetLength(MdbInfos, MdbFiles.Count);
    for i := 0 to Length(MdbInfos) - 1 do
    begin
    	MdbInfos[i] := GetMdb(MdbFiles[i]);
        FileName := ExtractFileName(MdbFiles[i]);
        Time	 := IntToStr(MdbInfos[i].FinishTick - MdbInfos[i].StartTick) + 'ms';
        ShowForm(MakeDottedLine(FileName, Time, 30))
    end;
    ShowForm('*****͏I*****');

    ShowForm('*****HTMLo͊Jn*****');

    Works.Add('Generated by mdber20030201');
    Works.Add('<hr><ul>');
    for i := 0 to Length(MdbInfos) - 1 do
    begin
        Works.Add('<li><a href="#' + MdbInfos[i].Name + '">' + MdbInfos[i].Name + '</a>');
    end;
    Works.Add('</ul>');

    for i := 0 to Length(MdbInfos) - 1 do
    begin
        Works.Add(MdbToHTML(MdbInfos[i]));
        FileName := ExtractFileName(MdbFiles[i]);
        ShowForm(':' + FileName);
    end;
    SaveHTMLPage(Works.Text);
    ShowForm('*****HTMLo͏I*****');

    ShowForm('I');
    CoUnInitialize;		// COM̏I

    ShowForm(GetCurrentTime() + 'I܂B');
end;

(*
	=============================================
	procedure	TMdbSeek.ShowForm(AMsg: string);
	=============================================
*)
procedure	TMdbSeek.ShowForm(AMsg: string);
begin
	ProgressMsg := AMsg;
    Synchronize(SyncShowForm);
end;

(*
	=============================================
	procedure	TMdbSeek.SyncShowForm();
	=============================================
*)
procedure	TMdbSeek.SyncShowForm();
begin
	if Progress <> nil then
    begin
    	Progress.Add(ProgressMsg);
    end;
end;

end.
